// Lab17_Biquad.pde
// MUSC208 John Ellinger Winter2014
import oscP5.*;
import netP5.*;
import controlP5.*;

ControlP5 cp5;
// the controlP5 controls
Slider sliderGain;
Slider sliderFreq;
Slider sliderQ;
RadioButton filterBtns;
float None = 1.0;
float Lowpass = 2.0;
float Highpass = 3.0;
float Bandpass = 4.0;
float Bandreject = 5.0;
float Resonant = 6.0;
float filterType = 1;
float sliderMin = 60;
float sliderMax = 8000;
float freq = 1000.0;
float oldFreq = 0; // used to eliminate excess drawing
Boolean firstDraw = true;


// OscP5 is name of class, oscP5 is a variable instance of that class
OscP5 oscP5; 
// ChucK network address
// on the same computer it will be "127.0.0.1"
// or "localhost"
NetAddress ChuckAddress; 

// GUI control layout
void addGUIControls()
{
  // create the font used for for control labels
  // use true/false for anti-aliased/not anti-aliased
  PFont pfont = createFont("Arial", 14, true); 
  ControlFont font = new ControlFont (pfont );
  cp5.setControlFont( font );

  // slider dimensions
  int sliderX = 20;    // slider height
  int sliderY = 90;    // slider height
  int sliderHt = 20;    // slider height
  int sliderWid = 700;  // slider width
  int sliderOffsetY = 40;  // vertical spacing between sliders
  int lblY = 44;
  int lblOffset = 122;
  // control default colors
  int bkgColor = color(128);            // light gray
  int foreColor = color(255, 255, 162);   // light yellow
  int activeColor = color(255, 255, 162); // light yellow


  filterBtns = cp5.addRadioButton("filterBtns")
    .setPosition(20, 20)
      .setSize(80, 20)
        .setColorLabel( 0 )
          .setItemsPerRow(6)
            .setSpacingColumn(43)
              .setColorBackground( bkgColor )   // dark gray
                .setColorForeground( bkgColor )  // light yellow
                  .setColorActive( activeColor )    // light yellos
                    .setMouseOver(false) 
                      .setNoneSelectedAllowed(false)
                        .addItem("None", 1)
                          .addItem("Lowpass", 2)
                            .addItem("Highpass", 3)
                              .addItem("Bandpass", 4)
                                .addItem("Bandreject", 5)
                                  .addItem("Resonant", 6)
                                    ;
  filterBtns.hideLabels();
  cp5.addTextlabel("NoneLabel").setText("None").setColor(0).setPosition(20+0*lblOffset, lblY);
  cp5.addTextlabel("LowpassLabel").setText("Lowpass").setColor(0).setPosition(20+1*lblOffset, lblY);
  cp5.addTextlabel("HighassLabel").setText("Highpass").setColor(0).setPosition(20+2*lblOffset, lblY);
  cp5.addTextlabel("BandpassLabel").setText("Bandpass").setColor(0).setPosition(20+3*lblOffset, lblY);
  cp5.addTextlabel("BandrejectLabel").setText("Bandreject").setColor(0).setPosition(20+4*lblOffset, lblY);
  cp5.addTextlabel("ResonantLabel").setText("Resonant").setColor(0).setPosition(20+5*lblOffset, lblY);

  // create slider Gain
  sliderGain = cp5.addSlider( "Gain" ) // Gain is the control label
    .setPosition( sliderX, sliderY + 0 * sliderOffsetY )              // 20 pixels in from left, 16 down from top
      .setSize( sliderWid, sliderHt )   // 480 pixels wide by 20 pixels tall
        .setRange( 0, 10.0 )                 // slider minimum value is 0, max is 1.0
            .setValue( 1.0 )                  // initial setting at 0.5
            .setColorCaptionLabel(0)          // caption color is black
              .setColorValueLabel(0)            // label color is black
                .setDecimalPrecision(2)           // display 2 decimal places
                  .setColorBackground( bkgColor )   // dark gray
                    .setColorForeground( foreColor )  // light yellow
                      .setColorActive( activeColor )    // light yellos
                        ;
  // change control label font and size
  cp5.getController("Gain").getCaptionLabel().toUpperCase(false);

  sliderFreq = cp5.addSlider( "Frequency" ) // a0 is the control label
    .setPosition( sliderX, sliderY + 1 * sliderOffsetY )             // XX means you figure it out
      .setSize( sliderWid, sliderHt )   // 480 pixels wide by 20 pixels tall
        .setRange( sliderMin, sliderMax )                 // slider minimum value is 0, max is 1.0
          .setValue( freq )                  // initial setting at 0.5
            .setColorCaptionLabel(0)          // caption color is black
              .setColorValueLabel(0)            // label color is black
                .setDecimalPrecision(0)           // display 2 decimal places
                  .setColorBackground( bkgColor )   // dark gray
                    .setColorForeground( foreColor )  // light yellow
                      .setColorActive( activeColor )    // light yellow
                        ;
  // change control label font and size
  cp5.getController("Frequency").getCaptionLabel().toUpperCase(false);

  sliderQ = cp5.addSlider( "Q" ) // a0 is the control label
    .setPosition( sliderX, sliderY + 2 * sliderOffsetY )             // XX means you figure it out
      .setSize( sliderWid, sliderHt )   // 480 pixels wide by 20 pixels tall
        .setRange( .01, 20 )                 // slider minimum value is 0, max is 1.0
            .setValue( 0.01 )                  // initial setting at 0.5
            .setColorCaptionLabel(0)          // caption color is black
              .setColorValueLabel(0)            // label color is black
                .setDecimalPrecision(2)           // display 2 decimal places
                  .setColorBackground( bkgColor )   // dark gray
                    .setColorForeground( foreColor )  // light yellow
                      .setColorActive( activeColor )    // light yellos
                        ;
  // change control label font and size
  cp5.getController("Q").getCaptionLabel().toUpperCase(false);
} // end addGUIControls

void setup() 
{
  // set size of window
  // MUST BE first line in setup
  size( 810, 220 );           // window will be 640 pixels wide by 18 pixels tall
  noStroke();                 // no aliasing
  cp5 = new ControlP5(this);  // create the cp5 object
  addGUIControls();           // add the controls

    // OSC stuff
  // if ChucK were to send back to Processing it would send to port 12345
  oscP5 = new OscP5(this, 12345); 
  // Processing will send to ChucK on port 12346
  // 127.0.0.1 is the network address of the local machine
  ChuckAddress = new NetAddress("127.0.0.1", 12346);
}

void draw() 
{
  // width and height are Processing variables passed to
  // the the size() function in in setup()
  rect(0, 0, width, height);    
  background( color(192) );  // a gray window background

  // activate None button
  if ( firstDraw )
  {
    filterBtns.activate(0);
    firstDraw = false;
  }
  // only draw when frequency changed
  if ( freq != oldFreq )
  {
    sliderFreq.setValue( freq );
    oldFreq = freq;
  }
}

void debugPrint( String s, float f )
{
  Boolean dbg = true;
  if ( dbg )
    println( s, f );
}

void sendOSCgain( float gane)
{
  // the OscMessage class has handy formatting methods
  OscMessage oscGain = new OscMessage("/lab17/gain");
  oscGain.add( gane );
  oscP5.send(oscGain, ChuckAddress);
  debugPrint( "OSC Gain", gane );
}

void sendOSCFrequency( float freq )
{
  //the OscMessage class has handy formatting methods
  OscMessage oscFreq = new OscMessage("/lab17/freq"); //greate the address
  oscFreq.add( freq );//append these items
  oscP5.send(oscFreq, ChuckAddress);//send the message
  debugPrint( "OSC Freq", freq );
}

void sendOSCQ( float q )
{
  //the OscMessage class has handy formatting methods
  OscMessage oscQ = new OscMessage("/lab17/q"); //greate the address
  oscQ.add( q );//append these items
  oscP5.send(oscQ, ChuckAddress);//send the message
  debugPrint( "Q", q );
}

void sendOSCFilterType( float filter ) 
{
  //the OscMessage class has handy formatting methods
  OscMessage oscFilterType = new OscMessage("/lab17/filterType");//greate the address
  oscFilterType.add( filter ); //append these items
  oscP5.send(oscFilterType, ChuckAddress);//send the message
  debugPrint( "Filter", filterType );
}

void controlEvent(ControlEvent theEvent)
{
  if (theEvent.isFrom( filterBtns ) ) 
  {
    filterType = theEvent.getValue();
    sendOSCFilterType( filterType );
    if ( filterType == 6 ) // Resonant
    {
      sliderGain.setRange(0.01, 0.5).setDecimalPrecision(2).setValue(0.051);
      sliderQ.setRange(0.9, 0.9999).setDecimalPrecision(4).setValue(0.9);
    }
    else
    {
      sliderGain.setRange(0, 10).setDecimalPrecision(2).setValue(1.0);
      sliderQ.setRange(0.01, 20).setDecimalPrecision(2).setValue(0.01);
    }
  }
  else if ( theEvent.isFrom( sliderGain ) )
  {
    float gane = sliderGain.getValue();
    sendOSCgain( gane );
  }
  else if ( theEvent.isFrom( sliderFreq ) )
  {
    // only allow multiples of 10.
    freq = sliderFreq.getValue();
    freq = round( freq / 10 ) * 10;
    Label lbl = sliderFreq.getValueLabel();
    lbl.setText( nf( freq, 2, 0 ) );
    sendOSCFrequency( freq );
  }
  else if ( theEvent.isFrom( sliderQ ) )
  {
    float q = sliderQ.getValue();
    sendOSCQ( q );
  }
}
