// simpleFIR.ck
// John Ellinger Music 208 Winter 2014

Impulse imp; // define but don't connect yet
SndBufUtilsClass sbc;
//sbc.init( me.sourceDir() + "/sines20K.wav" );
sbc.init( me.sourceDir() + "/noise.wav" );
//sbc.init( me.sourceDir() + "/StarsNStripes.wav" );

// copy the SndBuf samples from the SndBufUtilsClass
// into a local array of samples called ra.
sbc.getSamples() @=> float ra[];
ra.cap() => int numSamps;
// sanity check
<<< "numSamps", numSamps >>>;

// Set the gain to the initial Processing
0.5 => float gane; 
0.5 => float a0;
0.5 => float a1;
1.0 => float lowhi;
0.0 => float bypass;

// create our OSC receiver
OscRecv recv;
// listen on port 12346 that was defined in Processing
12346 => recv.port;
// start listening
recv.listen();

// listen for the /lab16/gain event followed by one float
recv.event( "/lab16/gain, f" ) @=> OscEvent oeGain;
// listen for the /lab16/a0 event followed by two floats
recv.event( "/lab16/a0, f f" ) @=> OscEvent oeA0;
recv.event( "/lab16/lowhi, f" ) @=> OscEvent oeLowHi;
recv.event( "/lab16/bypass, f" ) @=> OscEvent oeBypass;

// The OSC Listening function for sliderGain
function void updateGain() 
{
	while ( true )
	{
		// wait for event to arrive
		oeGain => now;
		// grab the next message from the queue
		// it will be one float - the sliderGain value
		while ( oeGain.nextMsg() != 0 )
		{ 
			oeGain.getFloat() => gane;
			<<< "sliderGain", gane >>>;
		}
	}
}

function void updateSliders() 
{
	while ( true )
	{
		// wait for event to arrive
		oeA0 => now;
		// grab the next message from the queue. 
		// it will be two floats - the values of a0 and a1
		while ( oeA0.nextMsg() != 0 )
		{ 
			oeA0.getFloat() => a0;
			oeA0.getFloat() => a1;
			<<< "a0", a0, "\ta1", a1  >>>;
		}
	}
}

function void updateLowPassHiPass() 
{
	while ( true )
	{
		// wait for event to arrive
		oeLowHi => now;
		// grab the next message from the queue. 
		while ( oeLowHi.nextMsg() != 0 )
		{ 
			oeLowHi.getFloat() => lowhi;
			if ( lowhi == 1.0 )
				<<< "low pass", lowhi  >>>;
			else if ( lowhi == -1.0 )
				<<< "high pass", lowhi  >>>;
			else
				<<< "bad pass" >>>;
		}
	}
}

function void updateBypass() 
{
	while ( true )
	{
		// wait for event to arrive
		oeBypass => now;
		// grab the next message from the queue. 
		while ( oeBypass.nextMsg() != 0 )
		{ 
			oeBypass.getFloat() => bypass;
			<<< "bypass", bypass  >>>;
		}
	}
}

// spork the OSC listener
spork ~ updateGain(); 
spork ~ updateSliders(); 
spork ~ updateLowPassHiPass(); 
spork ~ updateBypass();

// connect Impulse to dac
imp => dac;

// our counter
0 => int n;
while (true)
{
	if ( n == 0 )
		gane * ra[0] => imp.next;
	else 
		gane * ( (a0 * ra[n] ) + lowhi * (a1 * ra[n-1] ) ) => imp.next;
	
	if ( bypass )
		gane * ra[n] => imp.next;
	
	1::samp => now;
	n++;
	if ( n == numSamps )
		0 => n;
}
